---
jupytext:
  formats: ipynb,md:myst
  text_representation:
    extension: .md
    format_name: myst
    format_version: 0.13
    jupytext_version: 1.14.1
kernelspec:
  display_name: Python 3 (ipykernel)
  language: python
  name: python3
---

# List Comprehension Nachklapp

Nachklap zum Thema List Comprehension.

```{code-cell} ipython3
from timeit import default_timer as timer
```

```{code-cell} ipython3
result = [] # da sammeln wir unsere Ergebnisse hinein

start = timer()
for aussen in range(1000):
    for innen in range(20):
        if aussen in [1, 2]:
            if innen % 5 == 0:
                result.append(aussen*innen)
end = timer()
print(end - start)

print(result)
```

```{code-cell} ipython3
start = timer()
[  aussen*innen for aussen in range(1000) for innen in range(20) if aussen in [1,2] if innen % 5 == 0]
end = timer()
print(end - start)

print(result)
```

```{code-cell} ipython3
result = []
for aussen in range(1000):
    for innen in range(20):
        if aussen in [1,2]:
            if innen % 5 == 0:
                result.append(aussen*innen)
result
```

## dict comprehensions

Wir benötigen ein dict von Quadratzahlen. Wir erzeugen eines, indem wir durch einen Range durchlaufen.

```{code-cell} ipython3
quadrate = { x: x**2  for x in range(10)  }
quadrate
```

Ab jetzt interessiert uns dieses dict.

```{code-cell} ipython3
# alle keys, indem wir durch ein dict durchlaufen
q = { x for x  in quadrate }
q, type(q)
```

```{code-cell} ipython3
# das gleiche: nochmal alle keys, indem wir durch ein dict durchlaufen
q = { x for x  in quadrate.keys() }
q, type(q)
```

```{code-cell} ipython3
# alle Values, indem wir durch ein dict durchlaufen
q = { x for x  in quadrate.values() }
q, type(q)
```

```{code-cell} ipython3
# alle key-value-Tupel, indem wir durch ein dict durchlaufen
q = { x for x  in quadrate.items() }
print(q, type(q))
```

```{code-cell} ipython3
# alle key-value-Tupel, indem wir durch ein dict durchlaufen
q = { (x,y) for (x,y)  in quadrate.items() }
print(q, type(q))
```

```{code-cell} ipython3
# alle key-value-Tupel, indem wir durch ein dict durchlaufen
q = { (x,y) for x,y in quadrate.items() }
print(q, type(q))
```

```{code-cell} ipython3
# das geht nicht
# q = { x,y for x,y in quadrate.items() }
# print(q, type(q))
```

```{code-cell} ipython3
# alle key-value-Tupel, indem wir durch ein dict durchlaufen
q = { x: y for x,y in quadrate.items() }
print(q, type(q))
```

```{code-cell} ipython3
# alle key-value-Tupel, indem wir durch ein dict durchlaufen
q = { 2*x: 4*y for x,y in quadrate.items() }
print(q, type(q))
```

## Sternchen

```{code-cell} ipython3
def volumen_1(l, b, h, koerper):
    if koerper == "quader":
        return l * b * h
    elif koerper == "pyramide":
        return l * b * h / 3
    else:
        print(f"{koerper=} ist unbekannt")
```

```{code-cell} ipython3
volumen_1(10,10,10,"quader")
```

```{code-cell} ipython3
anderes_ding = [10,10,10,"pyramide"]
volumen_1(*anderes_ding)
```

```{code-cell} ipython3
def volumen_2(*, l, b, h, koerper):
    if koerper == "quader":
        return l * b * h
    elif koerper == "pyramide":
        return l * b * h / 3
```

```{code-cell} ipython3
# das geht nicht
# volumen_2(10,10,10,"quader")
```

```{code-cell} ipython3
volumen_2(koerper="quader", l=10, b=10, h=10)
```

```{code-cell} ipython3
ding = { "koerper": "quader", "l":10, "b": 10, "h":10 }
```

```{code-cell} ipython3
# umständlich
volumen_2(koerper = ding["koerper"], l = ding["l"],  h = ding["h"], b = ding["b"])
```

```{code-cell} ipython3
volumen_2(**ding)
```

```{code-cell} ipython3
def volumen_3(l, b, h, *, koerper="quader"):
    if koerper == "quader":
        return l * b * h
    elif koerper == "pyramide":
        return l * b * h / 3
```

```{code-cell} ipython3
volumen_3(10,10,10, koerper="quader")
```

```{code-cell} ipython3
volumen_3(10,10,10)
```

```{code-cell} ipython3
volumen_3(10,10,10, koerper="pyramide")
```

## Spielen!

am Beispiel von unterschiedlichen Arten, durch einen String durchzulaufen

+++

gegeben: ein String, z.B. `s = "Hallo!"`
    
gesucht: mit wie vielen unterschiedlichen Arten kann man durch einen String Buchstabe für Buchstabe durchlaufen?

```{code-cell} ipython3
s = "Hallo!"
```

```{code-cell} ipython3
for x in list(s):
    print(x, end="_")
```

```{code-cell} ipython3
for x in s:
    print(x, end="_")
```

```{code-cell} ipython3
for x in range(len(s)):
    print(s[x], end="_")
```

```{code-cell} ipython3
for x in range(len(s)):
    new_s = s[x:x+1]
    print(new_s, end="_")
```

```{code-cell} ipython3
for x in s:
    print(f"{x}", end="_")
```

```{code-cell} ipython3
for x in s:
    print(f"{x}_", end="")
```

```{code-cell} ipython3

```
